﻿/*
 * Copyright (c) 2013. Tomas Vera.
 * 
 * DISCLAIMER : This project and its components is intended to be used as an
 * illustration of programming techniques. 
 * NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED. 
 * By executing all or part of any of the included code you are ACKNOWLEDGING 
 * AND ASSUMING ANY AND ALL RISKS inherent with your actions.
*/

/*-----------------------------------------------------------------------------
 * IMPORTANT: You must run the "NestedXML.sql" script in your database
 * before you can run this application. The script is included in the 
 * bundle from which you retrieved this code sample.
 * -------------------------------------------------------------------------*/
using System;
using System.Data;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.Data.SqlClient;

namespace NestedXML
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            // Set the default state of our checkboxes
            cbNestedRelation.Checked = false;
            cbIncludeWrappers.Enabled = false;
        }


        /// <summary>
        /// This is the method that does all the work
        /// </summary>
        private void GetXML()
        {
            // This a connection string to our database (located in App.Config)
            string connStr = ConfigurationManager.ConnectionStrings["default"].ConnectionString;

            // Create our DB connection.
            SqlConnection conn = new SqlConnection(connStr);

            // This stored procedure returns four tables (Customer, Orders, 
            // OrderItems and CustNotes) [SEE NOTE AT TOP OF FILE]
            SqlCommand cmd = new SqlCommand("[sp_GetDataForXml]", conn);
            DataSet ds = new DataSet();

            // Use a DataAdapter to fill the dataset with four tables at once
            SqlDataAdapter a = new SqlDataAdapter(cmd);
            a.Fill(ds);

            // These tables are returned from our stored proc
            DataTable tblCustomers = ds.Tables[0];
            tblCustomers.TableName = "Customer";
            DataTable tblOrders = ds.Tables[1];
            tblOrders.TableName = "Order";
            DataTable tblOrderItems = ds.Tables[2];
            tblOrderItems.TableName = "OrderItem";
            DataTable tblNotes = ds.Tables[3];
            tblNotes.TableName = "CustNote";

            // For the purposes of this demo, we can choose to create or not
            // create the nested relations altogether.
            if (cbNestedRelation.Checked)
            {
                // We want to create relations. 
                // We have an "Include Wrappers" options to illustrate some 
                // programming options.
                if (cbIncludeWrappers.Checked)
                {
                    CreateNestedRelationsWithWrappers(ds);
                }
                else
                {
                    CreateNestedRelationsNoWrappers(ds);
                }
            }

            // Now display the XML generated by the dataset
            txtOutput.Text = ds.GetXml();
        }

        private void CreateNestedRelationsNoWrappers(DataSet ds)
        {

            // get some pointers to the tables in the dataset
            // (Makes it easer to illustrate what we're doing)
            DataTable tblCustomers = ds.Tables["Customer"];
            DataTable tblOrders = ds.Tables["Order"];
            DataTable tblOrderItems = ds.Tables["OrderItem"];
            DataTable tblNotes = ds.Tables["CustNote"];

            /* ---------------------------------------------------------------- 
            * Now, we need to explicitly define the Parent/Child 
            * relationship between all our tables
            * ---------------------------------------------------------------*/
            // This relation will yield an "Orders" node in each Customer node
            DataRelation relCustOrders = new DataRelation(
                "relCustOrders",                        // relation name
                tblCustomers.Columns["CustomerId"],     // parent column
                tblOrders.Columns["CustomerId"]); // child column

            // This will create an "Items" node under each Order node
            DataRelation relOrderItems = new DataRelation(
                "relOrderItems",                        // relation name
                tblOrders.Columns["OrderId"],           // parent column
                tblOrderItems.Columns["OrderId"]);     // child column

            // This will bundle all the notes under the "Notes" node
            DataRelation relCustNotes = new DataRelation(
                "relCustNotes",                         // relation name
                tblCustomers.Columns["CustomerId"],  // parent column
                tblNotes.Columns["CustomerId"]);        // child column

            // Set the "Nested" property on all the relations we created
            // Otherwise, this whole excercise is for naught.
            relCustOrders.Nested = cbNestedRelation.Checked;
            relOrderItems.Nested = cbNestedRelation.Checked;
            relCustNotes.Nested = cbNestedRelation.Checked;

            // Lastly, add the relations to the dataset
            ds.Relations.Add(relCustOrders);
            ds.Relations.Add(relOrderItems);
            ds.Relations.Add(relCustNotes);
        }

        /// <summary>
        /// Adds nested relations to our dataset with "wrappers" which allows
        /// grouping of child objects inside a "wrapper" node
        /// </summary>
        /// <param name="ds">The dataset to be updated</param>
        private void CreateNestedRelationsWithWrappers(DataSet ds)
        {
            // get some pointers to the tables in the dataset
            // (Makes it easer to illustrate what we're doing)
            DataTable tblCustomers = ds.Tables["Customer"];
            DataTable tblOrders = ds.Tables["Order"];
            DataTable tblOrderItems = ds.Tables["OrderItem"];
            DataTable tblNotes = ds.Tables["CustNote"];

            /*-----------------------------------------------------------------
            * Create some "Wrapper" tables  needed to group our child items 
            * in a nice bundles of nodes. 
            * ---------------------------------------------------------------*/
            // Will create a "Orders" node under each Customer
            DataTable tblOrderWrapper = new DataTable("Orders");
            tblOrderWrapper.Columns.Add("CustomerId",
                tblCustomers.Columns["CustomerId"].DataType);

            // Will Create a "Notes" node under each Customer
            DataTable tblNotesWrapper = new DataTable("Notes");
            tblNotesWrapper.Columns.Add("CustomerId",
                tblCustomers.Columns["CustomerId"].DataType);

            // This will create an "Items" node under each order
            DataTable tblItemWrapper = new DataTable("Items");
            tblItemWrapper.Columns.Add("OrderId",
                tblOrders.Columns["OrderId"].DataType);

            // Populate our "Orders" and "Notes" "wrapper" tables. 
            foreach (DataRow currCust in tblCustomers.Rows)
            {
                tblOrderWrapper.Rows.Add(currCust["CustomerId"]);
                tblNotesWrapper.Rows.Add(currCust["CustomerId"]);
            }
            // Populdate our "Items" wrapper table.
            foreach (DataRow currOrders in tblOrders.Rows)
            {
                tblItemWrapper.Rows.Add(currOrders["OrderId"]);
            }

            // Add our wrapper tables to the dataset
            ds.Tables.Add(tblOrderWrapper);
            ds.Tables.Add(tblItemWrapper);
            ds.Tables.Add(tblNotesWrapper);

            /* ---------------------------------------------------------------- 
            * Now, we need to explicitly define the Parent/Child 
            * relationship between all our tables
            * ---------------------------------------------------------------*/
            // This relation will yield an "Orders" node in each Customer node
            DataRelation relCustOrders = new DataRelation(
                "relCustOrders",                        // relation name
                tblCustomers.Columns["CustomerId"],     // parent column
                tblOrderWrapper.Columns["CustomerId"]); // child column

            // This will group all Orders inside the "Orders" node
            DataRelation relOrdersOrder = new DataRelation(
                "relOrdersOrder",                       // relation name
                tblOrderWrapper.Columns["CustomerId"],  // parent column
                tblOrders.Columns["CustomerId"]);       // child column

            // This will create an "Items" node under each Order node
            DataRelation relOrderItems = new DataRelation(
                "relOrderItems",                        // relation name
                tblOrders.Columns["OrderId"],           // parent column
                tblItemWrapper.Columns["OrderId"]);     // child column

            // This will bundle all the order items under the "Items" node
            DataRelation relItemWrapperItems = new DataRelation(
                "relItemWrapperItems",                  // relation name
                tblItemWrapper.Columns["OrderId"],      // parent column
                tblOrderItems.Columns["OrderId"]);      // Child column

            // This will create a "Notes" node under each Customer
            DataRelation relCustCustNotes = new DataRelation(
                "relCustCustsNotes",                    // relation name
                tblCustomers.Columns["CustomerId"],     // parent column
                tblNotesWrapper.Columns["CustomerId"]); // child column

            // This will bundle all the notes under the "Notes" node
            DataRelation relCustNotes = new DataRelation(
                "relCustNotes",                         // relation name
                tblNotesWrapper.Columns["CustomerId"],  // parent column
                tblNotes.Columns["CustomerId"]);        // child column

            // Set the "Nested" property on all the relations we created
            // Otherwise, this whole excercise is for naught.
            relCustOrders.Nested = cbNestedRelation.Checked;
            relOrdersOrder.Nested = cbNestedRelation.Checked;
            relOrderItems.Nested = cbNestedRelation.Checked;
            relItemWrapperItems.Nested = cbNestedRelation.Checked;
            relCustNotes.Nested = cbNestedRelation.Checked;
            relCustCustNotes.Nested = cbNestedRelation.Checked;

            // Lastly, add the relations to the dataset
            ds.Relations.Add(relCustOrders);
            ds.Relations.Add(relOrdersOrder);
            ds.Relations.Add(relOrderItems);
            ds.Relations.Add(relItemWrapperItems);
            ds.Relations.Add(relCustCustNotes);
            ds.Relations.Add(relCustNotes);
        }

        private void AppendReadmeToOutput()
        {
            string strReadMe = File.ReadAllText("_README.txt");

            txtOutput.AppendText("\r\n");
            txtOutput.AppendText("".PadRight(80, '-'));
            txtOutput.AppendText("\r\n");
            txtOutput.AppendText(strReadMe);
            txtOutput.AppendText("\r\n");

        }

        #region Event Handlers
        /// <summary>
        /// Event handler for the Click event on our "Get XML" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                GetXML();
            }
            catch (Exception ex)
            {
                txtOutput.Text = ex.Message;
                AppendReadmeToOutput();
                MessageBox.Show("There was a problem getting the XML for this demo. Ensure your database is properly configured.");
            }
        }


        /// <summary>
        /// Event handler to select All the output text.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectAllToolStripMenuItem_Click(object sender,EventArgs e)
        {
            txtOutput.SelectAll();
        }

        /// <summary>
        /// Handle "CheckedChanged" events on the "Nested Relations" checkbox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbNestedRelation_CheckedChanged(object sender, EventArgs e)
        {
            cbIncludeWrappers.Enabled = cbNestedRelation.Checked;

            // Clear the checkbox if IncludeWrappers is NOT checked
            if (!cbNestedRelation.Checked)
            {
                cbIncludeWrappers.Checked = cbNestedRelation.Checked;
            }
        }

        #endregion
    }
}


